@@ -34,17 +34,21 @@ Basic bot system designed to make it easy to do simple bots, optionally
 forking longer processes (like searches) concurrently in the background.
 
 There are several examples of bots using Bot::BasicBot in the examples/
-folder in the Bot::BasicBot tarball. If you installed Bot::BasicBot through
-CPAN, see http://jerakeen.org/programming/Bot-BasicBot for more docs and
-examples.
+folder in the Bot::BasicBot tarball.
 
 A quick summary, though - You want to define your own package that
 subclasses Bot::BasicBot, override various methods (documented below),
-then call new() and run() on it.
+then call L<C<new>|/new> and L<C<run>|/run> on it.
 
 =cut
 
 package Bot::BasicBot;
+BEGIN {
+  $Bot::BasicBot::AUTHORITY = 'cpan:HINRIK';
+}
+BEGIN {
+  $Bot::BasicBot::VERSION = '0.84';
+}
 
 use strict;
 use warnings;
@@ -56,21 +60,18 @@ use POE::Session;
 use POE::Wheel::Run;
 use POE::Filter::Line;
 use POE::Component::IRC;
+use POE::Component::IRC::Plugin::Connector;
 use Data::Dumper;
 use Text::Wrap ();
 
-our $VERSION = 0.81;
-
 use base qw( Exporter );
 our @EXPORT  = qw( say emote );
 
-our $RECONNECT_TIMEOUT = 500;
-
 =head1 STARTING THE BOT
 
-=head2 new( key => value, .. )
+=head2 new
 
-Creates a new instance of the class.  Name value pairs may be passed
+Creates a new instance of the class. Key/value pairs may be passed
 which will have the same effect as calling the method of that name
 with the value supplied. Returns a Bot::BasicBot object, that you can
 call 'run' on later.
@@ -124,8 +125,6 @@ sub run {
                 irc_msg          => "irc_said_state",
                 irc_public       => "irc_said_state",
                 irc_ctcp_action  => "irc_emoted_state",
-                irc_ping         => "irc_ping_state",
-                reconnect        => "reconnect",
 
                 irc_disconnected => "irc_disconnected_state",
                 irc_error        => "irc_error_state",
@@ -146,8 +145,7 @@ sub run {
                 irc_332          => "topic_raw_state",
                 irc_topic        => "topic_state",
 
-                irc_391          => "_time_state",
-                _get_time        => "_get_time_state",
+                irc_shutdown     => "shutdown_state",
                 
                 tick => "tick_state",
             }
@@ -159,15 +157,27 @@ sub run {
 
     # run
     $poe_kernel->run() unless $self->{no_run};
+    return;
 }
 
+=head1 STOPPING THE BOT
+
+To shut down the bot cleanly, use the L<C<shutdown>|/shutdown> method, which
+will (through L<C<AUTOLOAD>|/AUTOLOAD>) send an
+L<event|POE::Component::IRC/shutdown> of the same name to POE::Component::IRC,
+so it takes the same arguments:
+
+ $bot->shutdown( $bot->quit_message() );
+
+=cut
+
 =head1 METHODS TO OVERRIDE
 
 In your Bot::BasicBot subclass, you want to override some of the following
 methods to define how your bot works. These are all object methods - the
 (implicit) first parameter to all of them will be the bot object.
 
-=head2 init()
+=head2 init
 
 called when the bot is created, as part of new(). Override to provide
 your own init. Return a true value for a successful init, or undef if
@@ -175,10 +185,10 @@ you failed, in which case new() will die.
 
 =cut
 
-sub init { 1; }
+sub init { return 1; }
 
 
-=head2 said($args)
+=head2 said
 
 This is the main method that you'll want to override in your subclass -
 it's the one called by default whenever someone says anything that we
@@ -230,9 +240,9 @@ Returning undef will cause nothing to be said.
 
 =cut
 
-sub said { undef }
+sub said { return }
 
-=head2 emoted( $args )
+=head2 emoted
 
 This is a secondary method that you may wish to override. It gets called
 when someone in channel 'emotes', instead of talking. In its default
@@ -244,41 +254,41 @@ C<emoted> receives the same data hash as C<said>.
 =cut
 
 sub emoted {
-    shift->said(@_);
+    return shift->said(@_);
 }
 
-=head2 chanjoin( $mess )
+=head2 chanjoin
 
-Called when someone joins a channel. $mess is an object similar to a
-said() message, $mess->{who} is the nick of the user who joined,
-$mess->{channel} is the channel they joined.
+Called when someone joins a channel. It receives a hashref argument similar to
+the one received by said(). The key 'who' is the nick of the user who joined,
+while 'channel' is the channel they joined.
 
 This is a do-nothing implementation, override this in your subclass.
 
 =cut
 
-sub chanjoin { undef }
+sub chanjoin { return }
 
-=head2 chanpart( $mess )
+=head2 chanpart
 
-Called when someone leaves a channel. $mess is an object similar to a
-said() message, $mess->{who} is the nick of the user who left,
-$mess->{channel} is the channel they left.
+Called when someone joins a channel. It receives a hashref argument similar to
+the one received by said(). The key 'who' is the nick of the user who parted,
+while 'channel' is the channel they parted.
 
 This is a do-nothing implementation, override this in your subclass.
 
 =cut
 
-sub chanpart { undef }
+sub chanpart { return }
 
-=head2 got_names( $mess )
+=head2 got_names
 
 Whenever we have been given a definitive list of 'who is in the channel',
-this function will be called. As usual, $mess is a hash. $mess->{channel}
-will be the channel we have information for, $mess->{names} is a hashref,
-where the keys are the nicks of the users, and the values are more hashes,
-containing the two keys 'op' and 'voice', indicating if the user is a chanop
-or voiced respectively.
+this function will be called. It receives a hash reference as an argument.
+The key 'channel' will be the channel we have information for, 'names' is a
+hashref where the keys are the nicks of the users, and the values are more
+hashes, containing the two keys 'op' and 'voice', indicating if the user is
+a chanop or voiced respectively.
 
 The reply value is ignored.
 
@@ -290,21 +300,23 @@ method won't be called when that happens.
 
 =cut
 
-sub got_names { undef }
+sub got_names { return }
 
-=head2 topic( $mess )
+=head2 topic
 
-Called when the topic of the channel changes. $mess->{channel} is the channel
-the topic was set in, $mess->{who} is the nick of the user who changed the
-channel, and $mess->{topic} will be the new topic of the channel.
+Called when the topic of the channel changes. It receives a hashref argument.
+The key 'channel' is the channel the topic was set in, and 'who' is the nick
+of the user who changed the channel, 'topic' will be the new topic of the
+channel.
 
 =cut
 
-sub topic { undef }
+sub topic { return }
 
-=head2 nick_change( $mess )
+=head2 nick_change
 
-When a user changes nicks, this will be called. $mess looks like
+When a user changes nicks, this will be called. It receives a hashref which
+will look like this:
 
   { from => "old_nick",
     to => "new_nick",
@@ -312,11 +324,12 @@ When a user changes nicks, this will be called. $mess looks like
 
 =cut
 
-sub nick_change { undef }
+sub nick_change { return }
 
-=head2 kicked( $mess )
+=head2 kicked
 
-Called when a user is kicked from the channel. $mess looks like:
+Called when a user is kicked from the channel. It receives a hashref which
+will look like this:
 
   { channel => "#channel",
     who => "nick",
@@ -328,7 +341,7 @@ The reply value is ignored.
 
 =cut
 
-sub kicked { undef }
+sub kicked { return }
 
 =head2 tick()
 
@@ -360,7 +373,7 @@ whatsoever apart from returning this text.
 
 =cut
 
-sub help { "Sorry, this bot has no interactive help." }
+sub help { return "Sorry, this bot has no interactive help." }
 
 =head2 connected
 
@@ -369,11 +382,11 @@ to the server
 
 =cut
 
-sub connected { undef }
+sub connected { return }
 
-=head2 userquit( $mess )
+=head2 userquit
 
-$mess looks like
+Receives a hashref which will look like:
 
     { who => "nick that quit",
       body => "quit message",
@@ -383,21 +396,21 @@ $mess looks like
 
 sub userquit {
     my ($self, $mess) = @_;
+    return;
 }
 
 
-
 =head1 BOT METHODS
 
 There are a few methods you can call on the bot object to do things. These
 are as follows:
 
-=head2 schedule_tick(time)
+=head2 schedule_tick
 
-Causes the L<tick> event to be called in 'time' seconds (or 5 seconds if
-time is left unspecified). Note that if the tick event is due to be
-called already, this will override it, you can't schedule multiple
-future events with this funtction.
+Takes an integer as an argument. Causes the L<tick> event to be called after
+that many seconds (or 5 seconds if no argument is provided). Note that if the
+tick event is due to be called already, this will override it. You can't
+schedule multiple future events with this funtction.
 
 =cut
 
@@ -405,6 +418,7 @@ sub schedule_tick {
   my $self = shift;
   my $time = shift || 5;
   $self->{kernel}->delay( tick => $time );
+  return;
 }
 
 =head2 forkit
@@ -480,7 +494,7 @@ sub forkit {
         $args = \%args;
     }
 
-    return undef unless $args->{run};
+    return unless $args->{run};
 
     $args->{handler}   = $args->{handler}   || "_fork_said";
     $args->{arguments} = $args->{arguments} || [];
@@ -518,7 +532,7 @@ sub forkit {
             address => $args->{address}
         }
     };
-    return undef;
+    return;
 }
 
 sub _fork_said {
@@ -530,11 +544,13 @@ sub _fork_said {
     $args->{body} = $body;
 
     $self->say($args);
+    return;
 }
 
-=head2 say( key => value, .. )
+=head2 say
 
-Say something to someone.  You should pass the following arguments:
+Say something to someone. Takes a list of key/value pairs as arguments. You
+should pass the following arguments:
 
 =over 4
 
@@ -597,9 +613,12 @@ sub say {
     my $who = ( $args->{channel} eq "msg" ) ? $args->{who} : $args->{channel};
 
     unless ( $who && $body ) {
-        print STDERR "Can't PRIVMSG without target and body\n";
-        print STDERR " called from ".([caller]->[0])." line ".([caller]->[2])."\n";
-        print STDERR " who = '$who'\n body = '$body'\n";
+        $self->log( "Can't send a message without target and body\n"
+              . " called from "
+              . ( [caller]->[0] )
+              . " line "
+              . ( [caller]->[2] ) . "\n"
+              . " who = '$who'\n body = '$body'\n" );
         return;
     }
 
@@ -610,16 +629,23 @@ sub say {
     # I think the Text::Wrap docs lie - it doesn't do anything special
     # in list context
     my @bodies = split(/\n+/, $wrapped);
-    
+
+    # Allows to override the default "PRIVMSG". Used by notice()
+    my $irc_command = defined $args->{irc_command} && $args->{irc_command} eq 'notice'
+        ? 'notice'
+        : 'privmsg';
+
     # post an event that will send the message
     for my $body (@bodies) {
-        my ($who, $body) = $self->charset_encode($who, $body);
-        #warn "$who => $body\n";
-        $poe_kernel->post( $self->{IRCNAME}, 'privmsg', $who, $body );
+        my ($enc_who, $enc_body) = $self->charset_encode($who, $body);
+        #warn "$enc_who => $enc_body\n";
+        $poe_kernel->post( $self->{IRCNAME}, $irc_command, $enc_who, $enc_body );
     }
+
+    return;
 }
 
-=head2 emote( key => value, .. )
+=head2 emote
 
 C<emote> will return data to channel, but emoted (as if you'd said "/me
 writes a spiffy new bot" in most clients). It takes the same arguments
@@ -662,14 +688,59 @@ sub emote {
     # me too; i'll look at it in v0.5 - sb
 
     $poe_kernel->post( $self->{IRCNAME}, 'privmsg', $self->charset_encode($who, "\cAACTION " . $body . "\cA") );
+    return;
 }
 
-=head2 reply($mess, $body)
+=head2 notice
 
-Reply to a message $mess. Will reply to an incoming message
-with the text '$body', in a privmsg if $mess was a privmsg, in channel if
-not, and prefixes if $mess was prefixed. Mostly a shortcut method - it's
-roughly equivalent to $mess->{body} = $body; $self->say($mess);
+C<notice> will send a IRC notice to the channel. This is typically used by bots
+to not break the IRC conversations flow. The message will appear as:
+
+    -nick- message here
+
+It takes the same arguments as C<say>, listed above. Example:
+
+    $bot->notice(
+        channel => '#bot_basicbot_test',
+        body => 'This is a notice'
+    );
+
+=cut
+
+sub notice {
+
+    if ( !ref( $_[0] ) ) {
+        print $_[0] . "\n";
+        return 1;
+    }
+    
+    my $self = shift;
+    my $args;
+    if (ref($_[0])) {
+        $args = shift;
+    } else {
+        my %args = @_;
+        $args = \%args;
+    }
+
+    # Don't modify '$args' hashref in-place, or we might
+    # make all subsequent calls into notices
+    return $self->say(
+        %{ $args },
+        irc_command => 'notice'
+    );
+
+}
+
+=head2 reply
+
+Takes two arguments, a hashref containing information about an incoming
+message, and a reply message. It will reply in a privmsg if the incoming one
+was a privmsg, in channel if not, and with prefixes if the incoming one was
+prefixed. Mostly a shortcut method - it's roughly equivalent to
+
+ $mess->{body} = $body;
+ $self->say($mess);
 
 =cut
 
@@ -793,7 +864,7 @@ sub alt_nicks {
         my @args = ( ref $_[0] eq "ARRAY" ) ? @{ $_[0] } : @_;
         $self->{alt_nicks} = \@args;
     }
-    @{ $self->{alt_nicks} || [] };
+    return @{ $self->{alt_nicks} || [] };
 }
 
 =head2 username
@@ -806,7 +877,7 @@ will be the same as our nick.
 sub username {
     my $self = shift;
     $self->{username} = shift if @_;
-    $self->{username} or $self->nick;
+    return defined $self->{username} ? $self->{username} : $self->nick;
 }
 
 =head2 name
@@ -819,7 +890,7 @@ The name that the bot will identify itself as.  Defaults to
 sub name {
     my $self = shift;
     $self->{name} = shift if @_;
-    $self->{name} or $self->nick . " bot";
+    return defined $self->{name} ? $self->{name} : $self->nick . " bot";
 }
 
 =head2 channels
@@ -836,7 +907,7 @@ sub channels {
         my @args = ( ref $_[0] eq "ARRAY" ) ? @{ $_[0] } : @_;
         $self->{channels} = \@args;
     }
-    @{ $self->{channels} || [] };
+    return @{ $self->{channels} || [] };
 }
 
 =head2 quit_message
@@ -848,7 +919,7 @@ The quit message.  Defaults to "Bye".
 sub quit_message {
     my $self = shift;
     $self->{quit_message} = shift if @_;
-    defined( $self->{quit_message} ) ? $self->{quit_message} : "Bye";
+    return defined $self->{quit_message} ? $self->{quit_message} : "Bye";
 }
 
 =head2 ignore_list
@@ -866,7 +937,7 @@ sub ignore_list {
         my @args = ( ref $_[0] eq "ARRAY" ) ? @{ $_[0] } : @_;
         $self->{ignore_list} = \@args;
     }
-    @{ $self->{ignore_list} || [] };
+    return @{ $self->{ignore_list} || [] };
 }
 
 =head2 charset
@@ -897,7 +968,7 @@ Set to '1' to disable the built-in flood protection of POE::Compoent::IRC
 sub flood {
     my $self = shift;
     $self->{flood} = shift if @_;
-    $self->{flood};
+    return $self->{flood};
 }
 
 =head1 STATES
@@ -919,41 +990,10 @@ sub start_state {
 
     # Make an alias for our session, to keep it from getting GC'ed.
     $kernel->alias_set($self->{ALIASNAME});
-
-    $kernel->delay('reconnect', 1 );
-
     $kernel->delay('tick', 30);
-}
 
-=head2 reconnect
-
-Connects the bot to the IRC server. Called 1 second after the 'start'
-event.
-
-in an ideal world, this will never get called again - we schedule it for 'x'
-seconds in the future, and whenever we see a server ping we reset this
-counter again. This means that it'll get run if we haven't seen anything
-from the server for a while, so we can assume that something bad has
-happened. At that point we shotgun the IRC session and restart
-everything, so we reconnect to the server.
-
-This is by far the most reliable way I have found of ensuring that a bot
-will reconnect to a server after it's lost a network connection for some
-reason.
-
-By default, the timeout is 300 seconds. It can be set by changing
-$Bot::BasicBot::RECONNECT_TIMEOUT.
-
-=cut
-
-sub reconnect {
-    my ( $self, $kernel, $session ) = @_[ OBJECT, KERNEL, SESSION ];
-
-    $self->log("Trying to connect to server ".$self->server);
-
-    $kernel->call( $self->{IRCNAME}, 'disconnect' );
-    $kernel->call( $self->{IRCNAME}, 'shutdown' );
-    POE::Component::IRC->spawn( alias => $self->{IRCNAME} );
+    $self->{IRCOBJ} = POE::Component::IRC->spawn( alias => $self->{IRCNAME} );
+    $self->{IRCOBJ}->plugin_add('Connector', POE::Component::IRC::Plugin::Connector->new());
     $kernel->post( $self->{IRCNAME}, 'register', 'all' );
 
     $kernel->post($self->{IRCNAME}, 'connect',
@@ -972,8 +1012,8 @@ sub reconnect {
             ),
         }
     );
-    $kernel->delay('reconnect', $RECONNECT_TIMEOUT);
-    $kernel->delay('_get_time', 60);
+
+    return;
 }
 
 =head2 stop_state
@@ -982,12 +1022,7 @@ Called when we're stopping.  Shutdown the bot correctly.
 
 =cut
 
-sub stop_state {
-    my ( $self, $kernel ) = @_[ OBJECT, KERNEL ];
-
-    $kernel->post( $self->{IRCNAME}, 'quit', $self->charset_encode($self->quit_message) );
-    $kernel->alias_remove($self->{ALIASNAME});
-}
+sub stop_state { }
 
 =head2 irc_001_state
 
@@ -1013,32 +1048,31 @@ sub irc_001_state {
     $self->schedule_tick(5);
 
     $self->connected();
+    return;
 }
 
 =head2 irc_disconnected_state
 
-Called if we are disconnected from the server. 
-Logs the error and schedules a reconnect event.
+Called if we are disconnected from the server. Logs the error.
 
 =cut
 
 sub irc_disconnected_state {
     my ( $self, $kernel, $server ) = @_[ OBJECT, KERNEL, ARG0 ];
     $self->log("Lost connection to server $server.\n");
-    $kernel->delay('reconnect', 30);
+    return;
 }
 
 =head2 irc_error_state
 
-Called if there is an irc server error.
-Logs the error and schedules a reconnect event.
+Called if there is an irc server error. Logs the error.
 
 =cut
 
 sub irc_error_state {
     my ( $self, $err, $kernel ) = @_[ OBJECT, ARG0, KERNEL ];
     $self->log("Server error occurred! $err\n");
-    $kernel->delay('reconnect', 30);
+    return;
 }
 
 =head2 irc_kicked_state
@@ -1056,6 +1090,7 @@ sub irc_kicked_state {
     my $nick = $self->nick_strip($nickstring);
     $_[OBJECT]->_remove_from_channel( $channel, $kicked );
     $self->kicked({ channel => $channel, who => $nick, kicked => $kicked, reason => $reason });
+    return;
 }
 
 =head2 irc_join_state
@@ -1066,6 +1101,7 @@ Called if someone joins.  Used for nick tracking
 
 sub irc_join_state {
     my ( $self, $nick ) = @_[ OBJECT, ARG0 ];
+    return;
 }
 
 =head2 irc_nick_state
@@ -1082,12 +1118,14 @@ sub irc_nick_state {
           = delete $self->{channel_data}{$channel}{$nick};
     }
     $self->nick_change($nick, $newnick);
+    return;
 }
 
 =head2 irc_mode_state
 
 =cut
 
+## no critic (ControlStructures::ProhibitCascadingIfElse)
 sub irc_mode_state {
     my ($self, $kernel, $session) = @_[OBJECT, KERNEL, SESSION];
     my ($nickstring, $channel, $mode, @ops) = @_[ARG0..$#_];
@@ -1100,23 +1138,24 @@ sub irc_mode_state {
 
         my $op = shift(@modes);
 
-        if ($added and $op eq 'o') {
+        if ($added && $op eq 'o') {
           $current->{op} = 1;
           $current->{voice} = 0;
 
-        } elsif ($added and $op eq 'v') {
+        } elsif ($added && $op eq 'v') {
           $current->{voice} = 1 unless $current->{op};
 
-        } elsif (!$added and $op eq 'o') {
+        } elsif (!$added && $op eq 'o') {
           $current->{op} = 0;
           $current->{voice} = 0;
 
-        } elsif (!$added and $op eq 'v') {
+        } elsif (!$added && $op eq 'v') {
           $current->{voice} = 1 unless $current->{op};
         }
 
         $self->{channel_data}{$channel}{$who} = $current;
     }
+    return;
 }
 
 
@@ -1135,6 +1174,7 @@ sub irc_quit_state {
     # do this second, so that the userquit implementor has a chance to see
     # which channels they left
     $self->_remove_from_all_channels( $nick );
+    return;
 }
 
 =head2 irc_said_state
@@ -1145,8 +1185,8 @@ formats it into a nicer format and calls 'said'
 =cut
 
 sub irc_said_state {
-    $_[KERNEL]->delay( 'reconnect', $RECONNECT_TIMEOUT );
     irc_received_state( 'said', 'say', @_ );
+    return;
 }
 
 =head2 irc_emoted_state
@@ -1158,8 +1198,8 @@ which deals with it as if it was a spoken phrase.
 =cut
 
 sub irc_emoted_state {
-    $_[KERNEL]->delay( 'reconnect', $RECONNECT_TIMEOUT );
     irc_received_state( 'emoted', 'emote', @_ );
+    return;
 }
 
 =head2 irc_received_state
@@ -1206,15 +1246,15 @@ sub irc_received_state {
 
     $mess->{body} = $body;
     unless ( $mess->{channel} eq "msg" ) {
-        my $nick = $self->nick;
+        my $own_nick = $self->nick;
 
-        if ( $mess->{body} =~ s/^(\Q$nick\E)\s*[:,-]?\s*//i ) {
+        if ( $mess->{body} =~ s/^(\Q$own_nick\E)\s*[:,-]?\s*//i ) {
           $mess->{address} = $1;
         }
 
-        foreach $nick ( $self->alt_nicks ) {
+        foreach my $alt_nick ( $self->alt_nicks ) {
             last if $mess->{address};
-            if ( $mess->{body} =~ s/^(\Q$nick\E)\s*[:,-]?\s*//i ) {
+            if ( $mess->{body} =~ s/^(\Q$alt_nick\E)\s*[:,-]?\s*//i ) {
               $mess->{address} = $1;
             }
         }
@@ -1237,35 +1277,16 @@ sub irc_received_state {
     ### what did we get back?
 
     # nothing? Say nothing then
-    return unless $return;
+    return unless defined $return;
 
     # a string?  Say it how we were addressed then
-    unless ( ref($return) ) {
+    unless ( ref($return) || !length($return) ) {
         $mess->{body} = $return;
         $self->$respond($mess);
         return;
     }
 }
 
-=head2 irc_ping_state
-
-The most reliable way I've found of doing auto-server-rejoin is to listen for
-pings. Every ping we get, we put off rejoining the server for another few mins.
-If we haven't heard a ping in a while, the rejoin code will get called.
-
-Recently, I've adapted this for servers that don't send pings very often,
-and reset the counter any time _anything_ interesting happens.
-
-You can change the amount of time the bot waits between events before calling
-a reconnect event by changing $Bot::BasicBot::RECONNECT_TIMEOUT to a value in
-seconds. The default is '500'.
-
-=cut
-
-sub irc_ping_state {
-    $_[KERNEL]->delay( 'reconnect', $RECONNECT_TIMEOUT );
-}
-
 =head2 irc_chanjoin_state
 
 Called if someone joins a channel.
@@ -1274,7 +1295,6 @@ Called if someone joins a channel.
 
 sub irc_chanjoin_state {
     my $self = $_[OBJECT];
-    $_[KERNEL]->delay( 'reconnect', $RECONNECT_TIMEOUT );
     my ($channel, $nick) = @_[ ARG1, ARG0 ];
     $nick = $_[OBJECT]->nick_strip($nick);
     if ($self->nick eq $nick) {
@@ -1284,6 +1304,7 @@ sub irc_chanjoin_state {
     }
     $_[OBJECT]->_add_to_channel( $channel, $nick );
     irc_chan_received_state( 'chanjoin', 'say', @_ );
+    return;
 }
 
 =head2 irc_chanpart_state
@@ -1294,7 +1315,6 @@ Called if someone parts a channel.
 
 sub irc_chanpart_state {
     my $self = $_[OBJECT];
-    $_[KERNEL]->delay( 'reconnect', $RECONNECT_TIMEOUT );
     my ($channel, $nick) = @_[ ARG1, ARG0 ];
     $nick = $_[OBJECT]->nick_strip($nick);
     if ($self->nick eq $nick) {
@@ -1304,6 +1324,7 @@ sub irc_chanpart_state {
     }
     $_[OBJECT]->_remove_from_channel( $channel, $nick );
     irc_chan_received_state( 'chanpart', 'say', @_ );
+    return;
 }
 
 =head2 irc_chan_received_state
@@ -1356,6 +1377,7 @@ from memory.
 sub fork_close_state {
     my ( $self, $wheel_id ) = @_[ 0, ARG0 ];
     delete $self->{forks}->{$wheel_id};
+    return;
 }
 
 =head2 fork_error_state
@@ -1379,6 +1401,7 @@ sub tick_state {
     my ( $self, $kernel, $heap ) = @_[ OBJECT, KERNEL, HEAP ];
     my $delay = $self->tick();
     $self->schedule_tick($delay) if $delay;
+    return;
 }
 
 =head2 names_state
@@ -1402,6 +1425,7 @@ sub names_state {
       voice => $voice,
     }
   }
+  return;
 }
 
 =head2 names_done_state
@@ -1417,7 +1441,8 @@ sub names_done_state {
   my $built = delete $self->{building_channel_data}{$channel};
   return unless $built;
   $self->{channel_data}{$channel} = $built;
-  $self->names({ channel => $channel, names => $built });
+  $self->got_names({ channel => $channel, names => $built });
+  return;
 }
 
 
@@ -1425,11 +1450,13 @@ sub _add_to_channel {
   my ($self, $channel, $nick, $ops) = @_;
   $ops ||= { op => 0, voice => 0 };
   $self->{channel_data}{$channel}{$nick} = $ops;
+  return;
 }
 
 sub _remove_from_channel {
   my ($self, $channel, $nick) = @_;
   delete $self->{channel_data}{$channel}{$nick};
+  return;
 }
 
 sub _remove_from_all_channels {
@@ -1439,6 +1466,7 @@ sub _remove_from_all_channels {
       $self->_remove_from_channel( $channel, $nick );
     }
   }
+  return;
 }
 
 =head2 topic_raw_state
@@ -1449,6 +1477,7 @@ sub topic_raw_state {
   my ($self, $kernel, $server, $raw) = @_[OBJECT, KERNEL, ARG0, ARG1];
   my ($channel, $topic) = split(/ :/, $raw, 2);
   $self->topic({ channel => $channel, who => undef, topic => $topic });
+  return;
 }
 
 =head2 topic_state
@@ -1460,30 +1489,30 @@ sub topic_state {
     = @_[OBJECT, KERNEL, ARG0, ARG1, ARG2];
   my $nick = $self->nick_strip($nickraw);
   $self->topic({ channel => $channel, who => $nick, topic => $topic });
+  return;
 }
 
+=head2 shutdown_state
 
-# the server can tell us what it thinks the time is. We use this as
-# a work-around for the 'broken' behaviour of freenode (it doesn't send
-# ping messages)
-sub _get_time_state {
-  my ($self, $kernel) = @_[OBJECT, KERNEL];
-  $_[KERNEL]->post( $self->{IRCNAME}, "time");
+=cut
+
+sub shutdown_state {
+    my ($kernel, $self) = @_[KERNEL, OBJECT];
+    $kernel->delay('tick');
+    $kernel->alias_remove($self->{ALIASNAME});
+    for my $fork (values %{ $self->{forks} }) {
+        $fork->{wheel}->kill();
+    }
+    return;
 }
-sub _time_state {
-  my ($self, $kernel) = @_[OBJECT, KERNEL];
-  $_[KERNEL]->delay( 'reconnect', $RECONNECT_TIMEOUT );
-  $_[KERNEL]->delay( '_get_time', $RECONNECT_TIMEOUT / 2 );
-}
-
-
 
 =head1 OTHER METHODS
 
 =head2 AUTOLOAD
 
 Bot::BasicBot implements AUTOLOAD for sending arbitrary states to the
-underlying L<POE::Component::IRC> compoment. So for a $bot object, sending
+underlying L<POE::Component::IRC|POE::Component::IRC> component. So for a
+C<$bot> object, sending
 
     $bot->foo("bar");
 
@@ -1498,6 +1527,7 @@ sub AUTOLOAD {
     our $AUTOLOAD;
     $AUTOLOAD =~ s/.*:://;
     $poe_kernel->post( $self->{IRCNAME}, $AUTOLOAD, $self->charset_encode(@_) );
+    return;
 }
 
 =head2 log
@@ -1515,6 +1545,7 @@ sub log {
       chomp $log_entry;
       print STDERR "$log_entry\n";
     }
+    return;
 }
 
 =head2 ignore_nick($nick)
@@ -1525,7 +1556,7 @@ the ignore list
 =cut
 
 sub ignore_nick {
-    local $_;
+    local $_ = undef;
     my $self = shift;
     my $nick = shift;
     return grep { $nick eq $_ } @{ $self->{ignore_list} };
@@ -1546,7 +1577,7 @@ sub nick_strip {
     return $nick;
 }
 
-=head2 charset_decode( foo, bar, baz )
+=head2 charset_decode
 
 Converts a string of bytes into a perl string, using the bot's L<charset>.
 (under perls before 5.8, just returns the thing it's passed.
@@ -1584,7 +1615,7 @@ sub charset_decode {
   return @r;
 }
 
-=head2 charset_encode( foo, bar, baz )
+=head2 charset_encode
 
 Converts a list of perl strings into a list of byte sequences, using
 the bot's charset. See L<charset_decode>.
@@ -1610,6 +1641,11 @@ sub charset_encode {
   return @r;
 }
 
+=head1 HELP AND SUPPORT
+
+If you have any questions or issues, you can drop by in #poe @ irc.perl.org,
+where I (Hinrik) am usually around.
+
 =head1 AUTHOR
 
 Tom Insam E<lt>tom@jerakeen.orgE<gt>
@@ -1630,27 +1666,14 @@ AUTOLOAD stuff, better interactive help, and a few API tidies.
 Maintainership for a while was in the hands of Simon Kent
 E<lt>simon@hitherto.netE<gt>. Don't know what he did. :-)
 
-I recieved patches for tracking joins and parts from Silver, sat on
-them for two months, and have finally applied them. Thanks, dude. He also
+I (Tom Insam) recieved patches for tracking joins and parts from Silver, sat
+on them for two months, and have finally applied them. Thanks, dude. He also
 sent me changes for the tick event API, which made sense.
 
-=head1 SYSTEM REQUIREMENTS
-
-Bot::BasicBot is based on POE, and really needs the latest version as
-of writing (0.22), since POE::Wheel::Run (used for forking) is still
-under development, and the interface recently changed. With earlier
-versions of POE, forking will not work, and the makefile process will
-carp if you have < 0.22. Sorry.
-
-You also need POE::Component::IRC.
+In November 2010, maintainership moved to Hinrik E<Ouml>rn SigurE<eth>sson.
 
 =head1 BUGS
 
-During the make, make test make install process, POE will moan about
-its kernel not being run. I'll try and gag it in future releases, but
-hey, release early, release often, and it's not a fatal error. It just
-looks untidy.
-
 Don't call your bot "0".
 
 Nick tracking blatantly doesn't work yet. In Progress.
